@@ -158,42 +158,41 @@ ranap_message *hnbgw_decode_ranap_co(struct msgb *ranap_msg)
 static int handle_rx_rua(struct osmo_fsm_inst *fi, struct msgb *ranap_msg)
 {
 	struct hnbgw_context_map *map = fi->priv;
+
+	/* If the FSM instance has already terminated, don't dispatch anything. */
+	if (fi->proc.terminating)
+		return 0;
+
 	if (!msg_has_l2_data(ranap_msg))
 		return 0;
 
-	/* See if it is a RAB Assignment Response message from RUA to SCCP, where we need to change the user plane
-	 * information, for RTP mapping via MGW, or GTP mapping via UPF. */
-	if (!map->is_ps) {
-		ranap_message *message = hnbgw_decode_ranap_co(ranap_msg);
-		if (message) {
-			LOGPFSML(fi, LOGL_DEBUG, "rx from RUA: RANAP %s\n",
-				 get_value_string(ranap_procedure_code_vals, message->procedureCode));
+	ranap_message *message = hnbgw_decode_ranap_co(ranap_msg);
+	if (message) {
+		LOGPFSML(fi, LOGL_DEBUG, "rx from RUA: RANAP %s\n",
+			 get_value_string(ranap_procedure_code_vals, message->procedureCode));
 
-			kpi_ranap_process_ul(map, message);
+		kpi_ranap_process_ul(map, message);
 
+		if (!map->is_ps) {
+			/* See if it is a RAB Assignment Response message from RUA to SCCP, where we need to change the user plane
+			 * information, for RTP mapping via MGW, or GTP mapping via UPF. */
 			switch (message->procedureCode) {
 			case RANAP_ProcedureCode_id_RAB_Assignment:
 				/* mgw_fsm_handle_rab_ass_resp() takes ownership of prim->oph and (ranap) message */
 				return mgw_fsm_handle_cs_rab_ass_resp(map, ranap_msg, message);
 			}
-		}
+		} else {
 #if ENABLE_PFCP
-	} else if (hnb_gw_is_gtp_mapping_enabled()) {
-		/* map->is_ps == true and PFCP is enabled in osmo-hnbgw.cfg */
-		ranap_message *message = hnbgw_decode_ranap_co(ranap_msg);
-		if (message) {
-			LOGPFSML(fi, LOGL_DEBUG, "rx from RUA: RANAP %s\n",
-				 get_value_string(ranap_procedure_code_vals, message->procedureCode));
-
-			kpi_ranap_process_ul(map, message);
-
-			switch (message->procedureCode) {
-			case RANAP_ProcedureCode_id_RAB_Assignment:
-				/* ps_rab_ass_fsm takes ownership of prim->oph and RANAP message */
-				return hnbgw_gtpmap_rx_rab_ass_resp(map, ranap_msg, message);
+			if (hnb_gw_is_gtp_mapping_enabled()) {
+				/* map->is_ps == true and PFCP is enabled in osmo-hnbgw.cfg */
+				switch (message->procedureCode) {
+				case RANAP_ProcedureCode_id_RAB_Assignment:
+					/* ps_rab_ass_fsm takes ownership of prim->oph and RANAP message */
+					return hnbgw_gtpmap_rx_rab_ass_resp(map, ranap_msg, message);
+				}
 			}
-		}
 #endif
+		}
 	}
 
 	/* It was not a RAB Assignment Response that needed to be intercepted. Forward as-is to SCCP. */
@@ -220,16 +219,14 @@ static int forward_ranap_to_rua(struct hnbgw_context_map *map, struct msgb *rana
 
 static void map_rua_init_action(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
-	struct hnbgw_context_map *map = fi->priv;
 	struct msgb *ranap_msg = data;
 
 	switch (event) {
 
 	case MAP_RUA_EV_RX_CONNECT:
+		/* not needed for RAB assignment scanning, but for KPI scanning */
+		handle_rx_rua(fi, ranap_msg);
 		map_rua_fsm_state_chg(MAP_RUA_ST_CONNECTED);
-		/* The Connect will never be a RAB Assignment response, so no need for handle_rx_rua() (which decodes
-		 * the RANAP message to detect a RAB Assignment response). Just forward to SCCP as is. */
-		map_sccp_dispatch(map, MAP_SCCP_EV_TX_DATA_REQUEST, ranap_msg);
 		return;
 
 	case MAP_RUA_EV_RX_DISCONNECT:
@@ -290,15 +287,21 @@ static void map_rua_connected_action(struct osmo_fsm_inst *fi, uint32_t event, v
 		return;
 
 	case MAP_RUA_EV_RX_DISCONNECT:
-		/* received Disconnect from RUA. forward any payload to SCCP, and change state. */
-		if (!map_sccp_is_active(map)) {
-			/* If, unlikely, the SCCP is already gone, changing to MAP_RUA_ST_DISCONNECTED frees the
-			 * hnbgw_context_map. Avoid a use-after-free. */
-			map_rua_fsm_state_chg(MAP_RUA_ST_DISCONNECTED);
-			return;
+		/* 3GPP TS 25.468 9.1.5: RUA has disconnected.
+		 * - Under normal conditions (cause=Normal) the RUA Disconnect contains a RANAP Iu-ReleaseComplete.
+		 *   On SCCP, the Iu-ReleaseComplete should still be forwarded as N-Data SCCP Data Form 1),
+		 *   and we will expect the CN to send an SCCP RLSD soon.
+		 * - Under error conditions, cause!=Normal and there's no RANAP message.
+		 *   In that case, we need to tear down the associated SCCP link towards CN,
+		 *   which in turn will tear down the upper layer Iu conn.
+		 */
+		if (msg_has_l2_data(ranap_msg)) {
+			/* Forward any payload to SCCP before Disconnect. */
+			handle_rx_rua(fi, ranap_msg);
+		} else {
+			map->rua_fi_ctx.rua_disconnect_err_condition = true;
 		}
 		map_rua_fsm_state_chg(MAP_RUA_ST_DISCONNECTED);
-		handle_rx_rua(fi, ranap_msg);
 		return;
 
 	case MAP_RUA_EV_HNB_LINK_LOST:
@@ -318,13 +321,13 @@ static void map_rua_connected_action(struct osmo_fsm_inst *fi, uint32_t event, v
 	}
 }
 
-static void map_rua_free_if_done(struct hnbgw_context_map *map, uint32_t sccp_event)
+static void map_rua_free_if_done(struct hnbgw_context_map *map, uint32_t sccp_event, void *ev_data)
 {
 	/* From RUA's POV, we can now free the hnbgw_context_map.
 	 * If SCCP is still active, tell it to disconnect -- in that case the SCCP side will call context_map_free().
 	 * If SCCP is no longer active, free this map. */
 	if (map_sccp_is_active(map))
-		map_sccp_dispatch(map, sccp_event, NULL);
+		map_sccp_dispatch(map, sccp_event, ev_data);
 	else
 		context_map_free(map);
 }
@@ -332,7 +335,7 @@ static void map_rua_free_if_done(struct hnbgw_context_map *map, uint32_t sccp_ev
 static void map_rua_disconnected_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 {
 	struct hnbgw_context_map *map = fi->priv;
-	map_rua_free_if_done(map, MAP_SCCP_EV_RAN_DISC);
+	map_rua_free_if_done(map, MAP_SCCP_EV_RAN_DISC, (void *)map->rua_fi_ctx.rua_disconnect_err_condition);
 }
 
 static void map_rua_disconnected_action(struct osmo_fsm_inst *fi, uint32_t event, void *data)
@@ -346,7 +349,7 @@ static void map_rua_disconnected_action(struct osmo_fsm_inst *fi, uint32_t event
 static void map_rua_disrupted_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 {
 	struct hnbgw_context_map *map = fi->priv;
-	map_rua_free_if_done(map, MAP_SCCP_EV_RAN_LINK_LOST);
+	map_rua_free_if_done(map, MAP_SCCP_EV_RAN_LINK_LOST, NULL);
 }
 
 void map_rua_fsm_cleanup(struct osmo_fsm_inst *fi, enum osmo_fsm_term_cause cause)
