@@ -241,11 +241,7 @@ static int handle_rx_sccp(struct osmo_fsm_inst *fi, struct msgb *ranap_msg)
 				if (hnb_gw_is_gtp_mapping_enabled()) {
 					LOGP(DMAIN, LOGL_DEBUG,
 					     "RAB Assignment: setting up GTP tunnel mapping via UPF %s\n",
-#ifdef PFCP_PRIVATE_STRUCTS
 					     osmo_sockaddr_to_str_c(OTC_SELECT, osmo_pfcp_cp_peer_get_remote_addr(g_hnbgw->pfcp.cp_peer)));
-#else
-					     osmo_sockaddr_to_str_c(OTC_SELECT, &g_hnbgw->pfcp.cp_peer->remote_addr));
-#endif
 					return hnbgw_gtpmap_rx_rab_ass_req(map, ranap_msg, message);
 				}
 				/* If no UPF is configured, directly forward the message as-is (no GTP mapping). */
@@ -269,11 +265,12 @@ static int handle_rx_sccp(struct osmo_fsm_inst *fi, struct msgb *ranap_msg)
 static void map_sccp_init_action(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct hnbgw_context_map *map = fi->priv;
-	struct msgb *ranap_msg = data;
+	struct msgb *ranap_msg = NULL;
 
 	switch (event) {
 
 	case MAP_SCCP_EV_TX_DATA_REQUEST:
+		ranap_msg = data;
 		/* In the INIT state, the first MAP_SCCP_EV_TX_DATA_REQUEST will be the RANAP message received from the
 		 * RUA Connect message. Send the SCCP CR and transition to WAIT_CC. */
 		if (tx_sccp_cr(fi, ranap_msg) == 0)
@@ -281,15 +278,22 @@ static void map_sccp_init_action(struct osmo_fsm_inst *fi, uint32_t event, void
 		return;
 
 	case MAP_SCCP_EV_RAN_LINK_LOST:
-	case MAP_SCCP_EV_RAN_DISC:
 	case MAP_SCCP_EV_USER_ABORT:
 	case MAP_SCCP_EV_CN_LINK_LOST:
+		ranap_msg = data;
 		/* No CR has been sent yet, just go to disconnected state. */
 		if (msg_has_l2_data(ranap_msg))
 			LOG_MAP(map, DLSCCP, LOGL_ERROR, "SCCP not connected, cannot dispatch RANAP message\n");
 		map_sccp_fsm_state_chg(MAP_SCCP_ST_DISCONNECTED);
 		return;
 
+	case MAP_SCCP_EV_RAN_DISC:
+		/* bool rua_disconnect_err_condition = !!data; */
+		/* 3GPP TS 25.468 9.1.5: RUA has disconnected.
+		 * In this state we didn't send an SCCP CR yet, so nothing to be torn down on CN side. */
+		map_sccp_fsm_state_chg(MAP_SCCP_ST_DISCONNECTED);
+		return;
+
 	case MAP_SCCP_EV_RX_RELEASED:
 		/* SCCP RLSD received from CN. This will never happen since we haven't even asked for a connection, but
 		 * for completeness: */
@@ -304,31 +308,41 @@ static void map_sccp_init_action(struct osmo_fsm_inst *fi, uint32_t event, void
 static void map_sccp_wait_cc_action(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct hnbgw_context_map *map = fi->priv;
-	struct msgb *ranap_msg = data;
+	struct msgb *ranap_msg = NULL;
 
 	switch (event) {
 
 	case MAP_SCCP_EV_RX_CONNECTION_CONFIRM:
+		ranap_msg = data;
 		map_sccp_fsm_state_chg(MAP_SCCP_ST_CONNECTED);
 		/* Usually doesn't but if the SCCP CC contained data, forward it to RUA */
 		handle_rx_sccp(fi, ranap_msg);
 		return;
 
 	case MAP_SCCP_EV_TX_DATA_REQUEST:
+		/* ranap_msg = data; */
 		LOGPFSML(fi, LOGL_ERROR, "Connection not yet confirmed, cannot forward RANAP to CN\n");
 		return;
 
 	case MAP_SCCP_EV_RAN_LINK_LOST:
-	case MAP_SCCP_EV_RAN_DISC:
 	case MAP_SCCP_EV_USER_ABORT:
 	case MAP_SCCP_EV_CN_LINK_LOST:
+		ranap_msg = data;
 		/* RUA connection was terminated. First wait for the CC before releasing the SCCP conn. */
 		if (msg_has_l2_data(ranap_msg))
 			LOGPFSML(fi, LOGL_ERROR, "Connection not yet confirmed, cannot forward RANAP to CN\n");
 		map->please_disconnect = true;
 		return;
 
+	case MAP_SCCP_EV_RAN_DISC:
+		/* bool rua_disconnect_err_condition = !!data; */
+		/* 3GPP TS 25.468 9.1.5: RUA has disconnected.
+		 * In this state we didn't send an SCCP CR yet, so nothing to be torn down on CN side. */
+		map->please_disconnect = true;
+		return;
+
 	case MAP_SCCP_EV_RX_RELEASED:
+		ranap_msg = data;
 		/* SCCP RLSD received from CN. This will never happen since we haven't even received a Connection
 		 * Confirmed, but for completeness: */
 		handle_rx_sccp(fi, ranap_msg);
@@ -353,26 +367,40 @@ static void map_sccp_connected_onenter(struct osmo_fsm_inst *fi, uint32_t prev_s
 
 static void map_sccp_connected_action(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
-	struct msgb *ranap_msg = data;
+	struct msgb *ranap_msg = NULL;
+	bool rua_disconnect_err_condition;
 
 	switch (event) {
 
 	case MAP_SCCP_EV_RX_DATA_INDICATION:
+		ranap_msg = data;
 		/* forward RANAP from SCCP to RUA */
 		handle_rx_sccp(fi, ranap_msg);
 		return;
 
 	case MAP_SCCP_EV_TX_DATA_REQUEST:
+		ranap_msg = data;
 		/* Someone (usually the RUA side) wants us to send a RANAP payload to CN via SCCP */
 		tx_sccp_df1(fi, ranap_msg);
 		return;
 
 	case MAP_SCCP_EV_RAN_DISC:
-		/* RUA has disconnected, and usually has sent an Iu-ReleaseComplete along with its RUA Disconnect. On
-		 * SCCP, the Iu-ReleaseComplete should still be forwarded as N-Data (SCCP Data Form 1), and we will
-		 * expect the CN to send an SCCP RLSD soon. */
-		map_sccp_fsm_state_chg(MAP_SCCP_ST_WAIT_RLSD);
-		tx_sccp_df1(fi, ranap_msg);
+		rua_disconnect_err_condition = !!data;
+		/* 3GPP TS 25.468 9.1.5: RUA has disconnected.
+		 * - Under normal conditions (cause=Normal) the RUA Disconnect
+		 *   contained a RANAP Iu-ReleaseComplete which we already
+		 *   handled here through MAP_SCCP_EV_TX_DATA_REQUEST.
+		 *   On SCCP, We will expect the CN to send an SCCP RLSD soon.
+		 * - Under error conditions, cause!=Normal and there was no RANAP message.
+		 *   In that case, we need to tear down the associated SCCP link towards CN,
+		 *   which in turn will tear down the upper layer Iu conn.
+		 */
+		if (rua_disconnect_err_condition) {
+			tx_sccp_rlsd(fi);
+			map_sccp_fsm_state_chg(MAP_SCCP_ST_DISCONNECTED);
+		} else {
+			map_sccp_fsm_state_chg(MAP_SCCP_ST_WAIT_RLSD);
+		}
 		return;
 
 	case MAP_SCCP_EV_RAN_LINK_LOST:
@@ -381,6 +409,7 @@ static void map_sccp_connected_action(struct osmo_fsm_inst *fi, uint32_t event,
 	case MAP_SCCP_EV_USER_ABORT:
 		/* The user is asking for disconnection, so there is no Iu Release in progress. Disconnect now. */
 	case MAP_SCCP_EV_CN_LINK_LOST:
+		ranap_msg = data;
 		/* The CN peer has sent a RANAP RESET, so the old link that this map ran on is lost */
 
 		/* There won't be any ranap_msg, but if a caller wants to dispatch a msg, forward it before
@@ -391,6 +420,7 @@ static void map_sccp_connected_action(struct osmo_fsm_inst *fi, uint32_t event,
 		return;
 
 	case MAP_SCCP_EV_RX_RELEASED:
+		ranap_msg = data;
 		/* The CN sends an N-Disconnect (SCCP Released) out of the usual sequence. Not what we expected, but
 		 * handle it. */
 		LOGPFSML(fi, LOGL_ERROR, "CN sends SCCP Released sooner than expected\n");
@@ -399,6 +429,7 @@ static void map_sccp_connected_action(struct osmo_fsm_inst *fi, uint32_t event,
 		return;
 
 	case MAP_SCCP_EV_RX_CONNECTION_CONFIRM:
+		ranap_msg = data;
 		/* Already connected. Unusual, but if there is data just forward it. */
 		LOGPFSML(fi, LOGL_ERROR, "Already connected, but received SCCP CC again\n");
 		handle_rx_sccp(fi, ranap_msg);
@@ -419,11 +450,12 @@ static void map_sccp_wait_rlsd_onenter(struct osmo_fsm_inst *fi, uint32_t prev_s
 
 static void map_sccp_wait_rlsd_action(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
-	struct msgb *ranap_msg = data;
+	struct msgb *ranap_msg = NULL;
 
 	switch (event) {
 
 	case MAP_SCCP_EV_RX_RELEASED:
+		ranap_msg = data;
 		/* The CN sends the expected SCCP RLSD.
 		 * Usually there is no data, but if there is just forward it.
 		 * Usually RUA is already disconnected, but let the RUA FSM decide about that. */
@@ -432,12 +464,13 @@ static void map_sccp_wait_rlsd_action(struct osmo_fsm_inst *fi, uint32_t event,
 		return;
 
 	case MAP_SCCP_EV_RX_DATA_INDICATION:
+		ranap_msg = data;
 		/* RUA is probably already disconnected, but let the RUA FSM decide about that. */
 		handle_rx_sccp(fi, ranap_msg);
 		return;
 
 	case MAP_SCCP_EV_TX_DATA_REQUEST:
-	case MAP_SCCP_EV_RAN_DISC:
+		ranap_msg = data;
 		/* Normally, RUA would already disconnected, but since SCCP is officially still connected, we can still
 		 * forward messages there. Already waiting for CN to send the SCCP RLSD. If there is a message, forward
 		 * it, and just continue to time out on the SCCP RLSD. */
@@ -445,6 +478,7 @@ static void map_sccp_wait_rlsd_action(struct osmo_fsm_inst *fi, uint32_t event,
 		return;
 
 	case MAP_SCCP_EV_RX_CONNECTION_CONFIRM:
+		ranap_msg = data;
 		/* Already connected. Unusual, but if there is data just forward it. */
 		LOGPFSML(fi, LOGL_ERROR, "Already connected, but received SCCP CC\n");
 		handle_rx_sccp(fi, ranap_msg);
@@ -453,6 +487,7 @@ static void map_sccp_wait_rlsd_action(struct osmo_fsm_inst *fi, uint32_t event,
 	case MAP_SCCP_EV_RAN_LINK_LOST:
 	case MAP_SCCP_EV_USER_ABORT:
 	case MAP_SCCP_EV_CN_LINK_LOST:
+	case MAP_SCCP_EV_RAN_DISC:
 		/* Stop waiting for RLSD, send RLSD now. */
 		tx_sccp_rlsd(fi);
 		map_sccp_fsm_state_chg(MAP_SCCP_ST_DISCONNECTED);
