@@ -446,7 +446,7 @@ static int hnbgw_tx_ue_register_acc_tmsi(struct hnb_context *hnb, HNBAP_UE_Ident
 static int hnbgw_rx_hnb_deregister(struct hnb_context *ctx, ANY_t *in)
 {
 	HNBAP_HNBDe_RegisterIEs_t ies;
-	HNBAP_Cause_t cause;
+	HNBAP_Cause_t cause = {};
 	int rc;
 
 	rc = hnbap_decode_hnbde_registeries(&ies, in);
@@ -475,7 +475,7 @@ static int hnbgw_rx_hnb_register_req(struct hnb_context *ctx, ANY_t *in)
 	char identity_str[256];
 	const char *cell_id_str;
 	struct timespec tp;
-	HNBAP_Cause_t cause;
+	HNBAP_Cause_t cause = {};
 	struct osmo_sockaddr cur_osa = {};
 	socklen_t len = sizeof(cur_osa);
 
@@ -579,7 +579,7 @@ static int hnbgw_rx_hnb_register_req(struct hnb_context *ctx, ANY_t *in)
 static int hnbgw_rx_ue_register_req(struct hnb_context *ctx, ANY_t *in)
 {
 	HNBAP_UERegisterRequestIEs_t ies;
-	HNBAP_Cause_t cause;
+	HNBAP_Cause_t cause = {};
 	char imsi[GSM23003_IMSI_MAX_DIGITS+1];
 	int rc;
 
@@ -648,7 +648,7 @@ free_and_return_rc:
 static int hnbgw_rx_ue_deregister(struct hnb_context *ctx, ANY_t *in)
 {
 	HNBAP_UEDe_RegisterIEs_t ies;
-	HNBAP_Cause_t cause;
+	HNBAP_Cause_t cause = {};
 	int rc;
 	uint32_t ctxid;
 
