@@ -166,6 +166,20 @@ int rua_tx_disc(struct hnb_context *hnb, int is_ps, uint32_t context_id,
 	return hnbgw_rua_tx(hnb, msg);
 }
 
+/* Send Disconnect to RUA without RANAP data */
+static void rua_tx_disc_conn_fail(struct hnb_context *hnb, bool is_ps, uint32_t context_id)
+{
+	RUA_Cause_t rua_cause = {
+		.present = RUA_Cause_PR_radioNetwork,
+		.choice.radioNetwork = RUA_CauseRadioNetwork_connect_failed,
+	};
+
+	LOG_HNBP(hnb->persistent, LOGL_INFO, "Tx RUA Disconnect\n");
+
+	if (rua_tx_disc(hnb, is_ps, context_id, &rua_cause, NULL, 0))
+		LOG_HNBP(hnb->persistent, LOGL_ERROR, "Failed to send Disconnect to RUA\n");
+}
+
 static struct value_string rua_procedure_code_names[] = {
 	{ RUA_ProcedureCode_id_Connect, "Connect" },
 	{ RUA_ProcedureCode_id_DirectTransfer, "DirectTransfer" },
@@ -228,7 +242,6 @@ static int rua_to_scu(struct hnb_context *hnb,
 	struct msgb *ranap_msg = NULL;
 	struct hnbgw_context_map *map = NULL;
 	bool is_ps;
-	int logl;
 
 	switch (cN_DomainIndicator) {
 	case RUA_CN_DomainIndicator_cs_domain:
@@ -253,7 +266,6 @@ static int rua_to_scu(struct hnb_context *hnb,
 	}
 
 	map = context_map_find_by_rua_ctx_id(hnb, context_id, is_ps);
-	logl = LOGL_ERROR;
 
 	switch (rua_procedure) {
 	case RUA_ProcedureCode_id_Connect:
@@ -270,6 +282,7 @@ static int rua_to_scu(struct hnb_context *hnb,
 			LOGHNB(hnb, DRUA, LOGL_ERROR,
 			       "Failed to create context map for %s: rx RUA %s with %u bytes RANAP data\n",
 			       is_ps ? "IuPS" : "IuCS", rua_procedure_code_name(rua_procedure), data ? len : 0);
+			rua_tx_disc_conn_fail(hnb, is_ps, context_id);
 			return -EINVAL;
 		}
 		break;
@@ -278,13 +291,19 @@ static int rua_to_scu(struct hnb_context *hnb,
 		/* For RUA Disconnect, do not spam the ERROR log. It is just a stray Disconnect, no harm done.
 		 * Context: some CN are known to rapidly tear down SCCP without waiting for RUA to disconnect gracefully
 		 * (IU Release Complete). Such CN would cause ERROR logging for each and every released context map. */
-		logl = LOGL_DEBUG;
-		/* fall thru */
+		if (!map) {
+			LOGHNB(hnb, DRUA, LOGL_DEBUG, "rx RUA %s for unknown RUA context %u\n",
+			       rua_procedure_code_name(rua_procedure), context_id);
+			return -EINVAL;
+		}
+		break;
+
 	default:
 		/* Any message other than Connect must have a valid RUA context */
 		if (!map) {
-			LOGHNB(hnb, DRUA, logl, "rx RUA %s for unknown RUA context %u\n",
+			LOGHNB(hnb, DRUA, LOGL_ERROR, "rx RUA %s for unknown RUA context %u\n",
 			       rua_procedure_code_name(rua_procedure), context_id);
+			rua_tx_disc_conn_fail(hnb, is_ps, context_id);
 			return -EINVAL;
 		}
 		break;
