@@ -1,6 +1,7 @@
 /* HNB-GW interface to quagga VTY */
 
 /* (C) 2016 by sysmocom s.f.m.c. GmbH <info@sysmocom.de>
+ * (C) 2024 by Harald Welte <laforge@gnumonks.org>
  * All Rights Reserved
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,6 +35,7 @@
 #include <osmocom/hnbgw/hnbgw_cn.h>
 #include <osmocom/hnbgw/context_map.h>
 #include <osmocom/hnbgw/tdefs.h>
+#include <osmocom/hnbgw/nft_kpi.h>
 #include <osmocom/sigtran/protocol/sua.h>
 #include <osmocom/sigtran/sccp_helpers.h>
 #include <osmocom/netif/stream.h>
@@ -53,6 +55,12 @@ DEFUN(cfg_hnbgw, cfg_hnbgw_cmd,
 	return CMD_SUCCESS;
 }
 
+static struct cmd_node hnb_node = {
+	HNB_NODE,
+	"%s(config-hnbgw-hnb)# ",
+	1,
+};
+
 static struct cmd_node iuh_node = {
 	IUH_NODE,
 	"%s(config-hnbgw-iuh)# ",
@@ -197,12 +205,14 @@ static void vty_dump_hnb_info(struct vty *vty, struct hnb_context *hnb)
 	struct hnbgw_context_map *map;
 	unsigned int map_count[2] = {};
 	unsigned int state_count[2][MAP_S_NUM_STATES + 1] = {};
+	unsigned long long sec;
 
 	vty_out(vty, "HNB ");
 	vty_out_ofd_addr(vty, hnb->conn? osmo_stream_srv_get_ofd(hnb->conn) : NULL);
 	vty_out(vty, " \"%s\"%s", hnb->identity_info, VTY_NEWLINE);
-	vty_out(vty, "    MCC %u MNC %u LAC %u RAC %u SAC %u CID %u SCTP-stream:HNBAP=%u,RUA=%u%s",
-		hnb->id.mcc, hnb->id.mnc, hnb->id.lac, hnb->id.rac, hnb->id.sac, hnb->id.cid,
+	vty_out(vty, "    MCC %s MNC %s LAC %u RAC %u SAC %u CID %u SCTP-stream:HNBAP=%u,RUA=%u%s",
+		osmo_mcc_name(hnb->id.plmn.mcc), osmo_mnc_name(hnb->id.plmn.mnc, hnb->id.plmn.mnc_3_digits),
+		hnb->id.lac, hnb->id.rac, hnb->id.sac, hnb->id.cid,
 		hnb->hnbap_stream, hnb->rua_stream, VTY_NEWLINE);
 
 	llist_for_each_entry(map, &hnb->map_list, hnb_list) {
@@ -211,12 +221,12 @@ static void vty_dump_hnb_info(struct vty *vty, struct hnb_context *hnb)
 	}
 	vty_dump_hnb_info__map_states(vty, "IuCS", map_count[0], state_count[0]);
 	vty_dump_hnb_info__map_states(vty, "IuPS", map_count[1], state_count[1]);
-}
 
-static void vty_dump_ue_info(struct vty *vty, struct ue_context *ue)
-{
-	vty_out(vty, "UE IMSI \"%s\" context ID %u HNB %s%s", ue->imsi, ue->context_id,
-		hnb_context_name(ue->hnb), VTY_NEWLINE);
+	sec = hnb_get_updowntime(hnb);
+	if (sec) {
+		vty_out(vty, " Iuh Uptime: %llu days %llu hours %llu min. %llu sec.%s",
+			OSMO_SEC2DAY(sec), OSMO_SEC2HRS(sec), OSMO_SEC2MIN(sec), sec % 60, VTY_NEWLINE);
+	}
 }
 
 #define SHOW_HNB_STR SHOW_STR "Display information about HNB\n"
@@ -263,18 +273,6 @@ DEFUN(show_one_hnb, show_one_hnb_cmd, "show hnb NAME ",
 	return CMD_SUCCESS;
 }
 
-DEFUN(show_ue, show_ue_cmd, "show ue all",
-      SHOW_STR "Display HNBAP information about UE\n" "All UE\n")
-{
-	struct ue_context *ue;
-
-	llist_for_each_entry(ue, &g_hnbgw->ue_list, list) {
-		vty_dump_ue_info(vty, ue);
-	}
-
-	return CMD_SUCCESS;
-}
-
 DEFUN(show_talloc, show_talloc_cmd, "show talloc", SHOW_STR "Display talloc info")
 {
 	talloc_report_full(g_hnbgw, stderr);
@@ -355,6 +353,19 @@ DEFUN(cfg_hnbgw_log_prefix, cfg_hnbgw_log_prefix_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_hnbgw_hnb_policy, cfg_hnbgw_hnb_policy_cmd,
+	"hnb-policy (accept-all|closed)",
+	"Configure the policy of which HNB connections to accept\n"
+	"Accept HNB of any identity\n"
+	"Accept only HNB whose identity is explicitly configured via VTY\n")
+{
+	if (!strcmp(argv[0], "accept-all"))
+		g_hnbgw->config.accept_all_hnb = true;
+	else
+		g_hnbgw->config.accept_all_hnb = false;
+	return CMD_SUCCESS;
+}
+
 DEFUN_DEPRECATED(cfg_hnbgw_max_sccp_cr_payload_len, cfg_hnbgw_max_sccp_cr_payload_len_cmd,
       "sccp cr max-payload-len <0-999999>",
       "Configure SCCP behavior\n"
@@ -799,6 +810,88 @@ DEFUN(cfg_config_apply_sccp, cfg_config_apply_sccp_cmd,
 	return CMD_SUCCESS;
 }
 
+#define HNB_STR "hNodeB specific configuration\n"
+
+DEFUN(cfg_hnbgw_hnb, cfg_hnbgw_hnb_cmd,
+      "hnb UMTS_CELL_ID",
+      HNB_STR
+      "Identity of hNodeB in xxx-yyy-Llac-Rrac-Ssac-Ccid format\n")
+{
+	struct umts_cell_id ucid;
+	struct hnb_persistent *hnbp;
+
+	if (umts_cell_id_from_str(&ucid, argv[0])) {
+		vty_out(vty, "%% Invalid UMTS_CELL_ID '%s'%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	hnbp = hnb_persistent_find_by_id(&ucid);
+	if (!hnbp)
+		hnbp = hnb_persistent_alloc(&ucid);
+	if (!hnbp) {
+		vty_out(vty, "%% Could not create HNB '%s'%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	vty->index = hnbp;
+	vty->node = HNB_NODE;
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_hnbgw_no_hnb, cfg_hnbgw_no_hnb_cmd,
+	"no hnb IDENTITY_INFO",
+	NO_STR "Remove configuration for specified hNodeB\n"
+	"Identity of hNodeB\n")
+{
+	struct umts_cell_id ucid;
+	struct hnb_persistent *hnbp;
+
+	if (umts_cell_id_from_str(&ucid, argv[0])) {
+		vty_out(vty, "%% Invalid UMTS_CELL_ID '%s'%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	hnbp = hnb_persistent_find_by_id(&ucid);
+	if (!hnbp) {
+		vty_out(vty, "%% Could not find any HNB for identity '%s'%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	hnb_persistent_free(hnbp);
+	return CMD_SUCCESS;
+}
+
+#define NFT_KPI_STR "Retrieve traffic counters from nftables\n"
+
+DEFUN(cfg_hnbgw_nft_kpi, cfg_hnbgw_nft_kpi_cmd,
+      "nft-kpi [TABLE_NAME]",
+      NFT_KPI_STR
+      "Set a custom nft table name to use, instead of 'osmo-hnbgw'\n")
+{
+	const char *set_table_name = NULL;
+	if (argc > 0)
+		set_table_name = argv[0];
+
+	if (vty->type == VTY_TERM)
+		vty_out(vty, "%% WARNING: nft configuration changes need a restart of osmo-hnbgw%s", VTY_NEWLINE);
+
+	g_hnbgw->config.nft_kpi.enable = true;
+	osmo_talloc_replace_string(g_hnbgw, &g_hnbgw->config.nft_kpi.table_name, set_table_name);
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_hnbgw_no_nft_kpi, cfg_hnbgw_no_nft_kpi_cmd,
+	"no nft-kpi",
+	NO_STR NFT_KPI_STR)
+{
+	if (vty->type == VTY_TERM)
+		vty_out(vty, "%% WARNING: nft configuration changes need a restart of osmo-hnbgw%s", VTY_NEWLINE);
+	g_hnbgw->config.nft_kpi.enable = false;
+	return CMD_SUCCESS;
+}
+
 #if ENABLE_PFCP
 
 static struct cmd_node pfcp_node = {
@@ -836,12 +929,33 @@ DEFUN(cfg_pfcp_local_addr, cfg_pfcp_local_addr_cmd,
 DEFUN(cfg_pfcp_local_port, cfg_pfcp_local_port_cmd,
       "local-port <1-65535>",
       "Local port for PFCP\n"
-      "IP port\n")
+      "UDP port\n")
 {
 	g_hnbgw->config.pfcp.local_port = atoi(argv[0]);
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_pfcp_netinst, cfg_pfcp_netinst_cmd,
+      "netinst (access|core) NAME",
+      "Add a Network Instance IE to all outgoing PFCP rule sets,"
+      " so that the UPF may choose the correct interface to open GTP tunnels on.\n"
+      "Set the Network Instance name for the access side (towards RAN).\n"
+      "Set the Network Instance name for the core side.\n"
+      "The Network Instance name as a dotted string, typically a domain name like 'ran23.example.com'."
+      " A matching osmo-upf.cfg could be: 'netinst' / 'add ran23.example.com 10.0.0.23'."
+      " See 3GPP TS 29.244 8.2.4.\n")
+{
+	const char *access_or_core = argv[0];
+	char **str;
+	if (!strcmp(access_or_core, "access"))
+		str = &g_hnbgw->config.pfcp.netinst.access;
+	else
+		str = &g_hnbgw->config.pfcp.netinst.core;
+	osmo_talloc_replace_string(g_hnbgw, str, argv[1]);
+	LOGP(DLPFCP, LOGL_NOTICE, "cfg: pfcp netinst %s %s\n", access_or_core, *str);
+	return CMD_SUCCESS;
+}
+
 #endif /* ENABLE_PFCP */
 
 DEFUN_DEPRECATED(cfg_hnbgw_timer_ps, cfg_hnbgw_timer_ps_cmd,
@@ -868,8 +982,15 @@ static void _config_write_cnpool(struct vty *vty, struct hnbgw_cnpool *cnpool)
 	cnpool_write_nri(vty, cnpool, false);
 }
 
+static void write_one_hnbp(struct vty *vty, const struct hnb_persistent *hnbp)
+{
+	vty_out(vty, " hnb %s%s", hnbp->id_str, VTY_NEWLINE);
+}
+
 static int config_write_hnbgw(struct vty *vty)
 {
+	const struct hnb_persistent *hnbp;
+
 	vty_out(vty, "hnbgw%s", VTY_NEWLINE);
 
 	if (g_hnbgw->config.plmn.mcc)
@@ -882,11 +1003,24 @@ static int config_write_hnbgw(struct vty *vty)
 
 	vty_out(vty, " log-prefix %s%s", g_hnbgw->config.log_prefix_hnb_id ? "hnb-id" : "umts-cell-id",
 		VTY_NEWLINE);
+
+	if (!g_hnbgw->config.accept_all_hnb)
+		vty_out(vty, " hnb-policy closed%s", VTY_NEWLINE);
+
 	osmo_tdef_vty_groups_write(vty, " ");
 
+	llist_for_each_entry(hnbp, &g_hnbgw->hnb_persistent_list, list)
+		write_one_hnbp(vty, hnbp);
+
 	_config_write_cnpool(vty, &g_hnbgw->sccp.cnpool_iucs);
 	_config_write_cnpool(vty, &g_hnbgw->sccp.cnpool_iups);
 
+	if (g_hnbgw->config.nft_kpi.enable)
+		vty_out(vty, " nft-kpi%s%s%s",
+			g_hnbgw->config.nft_kpi.table_name ? " " : "",
+			g_hnbgw->config.nft_kpi.table_name ? : "",
+			VTY_NEWLINE);
+
 	return CMD_SUCCESS;
 }
 
@@ -945,8 +1079,16 @@ static int config_write_hnbgw_pfcp(struct vty *vty)
 	vty_out(vty, " pfcp%s", VTY_NEWLINE);
 	if (g_hnbgw->config.pfcp.local_addr)
 		vty_out(vty, "  local-addr %s%s", g_hnbgw->config.pfcp.local_addr, VTY_NEWLINE);
+	if (g_hnbgw->config.pfcp.local_port)
+		vty_out(vty, "  local-port %u%s", g_hnbgw->config.pfcp.local_port, VTY_NEWLINE);
 	if (g_hnbgw->config.pfcp.remote_addr)
 		vty_out(vty, "  remote-addr %s%s", g_hnbgw->config.pfcp.remote_addr, VTY_NEWLINE);
+	if (g_hnbgw->config.pfcp.netinst.access
+	    && *g_hnbgw->config.pfcp.netinst.access)
+		vty_out(vty, "  netinst access %s%s", g_hnbgw->config.pfcp.netinst.access, VTY_NEWLINE);
+	if (g_hnbgw->config.pfcp.netinst.core
+	    && *g_hnbgw->config.pfcp.netinst.core)
+		vty_out(vty, "  netinst core %s%s", g_hnbgw->config.pfcp.netinst.core, VTY_NEWLINE);
 
 	return CMD_SUCCESS;
 }
@@ -974,6 +1116,7 @@ void hnbgw_vty_init(void)
 	install_element(HNBGW_NODE, &cfg_hnbgw_rnc_id_cmd);
 	install_element(HNBGW_NODE, &cfg_hnbgw_log_prefix_cmd);
 	install_element(HNBGW_NODE, &cfg_hnbgw_max_sccp_cr_payload_len_cmd);
+	install_element(HNBGW_NODE, &cfg_hnbgw_hnb_policy_cmd);
 
 	install_element(HNBGW_NODE, &cfg_hnbgw_iuh_cmd);
 	install_node(&iuh_node, config_write_hnbgw_iuh);
@@ -998,10 +1141,16 @@ void hnbgw_vty_init(void)
 	install_element(IUCS_NODE, &cfg_hnbgw_cnpool_remote_addr_cmd);
 	install_element(IUPS_NODE, &cfg_hnbgw_cnpool_remote_addr_cmd);
 
+	install_element(HNBGW_NODE, &cfg_hnbgw_hnb_cmd);
+	install_element(HNBGW_NODE, &cfg_hnbgw_no_hnb_cmd);
+	install_node(&hnb_node, NULL);
+
+	install_element(HNBGW_NODE, &cfg_hnbgw_nft_kpi_cmd);
+	install_element(HNBGW_NODE, &cfg_hnbgw_no_nft_kpi_cmd);
+
 	install_element_ve(&show_cnlink_cmd);
 	install_element_ve(&show_hnb_cmd);
 	install_element_ve(&show_one_hnb_cmd);
-	install_element_ve(&show_ue_cmd);
 	install_element_ve(&show_talloc_cmd);
 
 	install_element(HNBGW_NODE, &cfg_hnbgw_mgcp_cmd);
@@ -1017,6 +1166,7 @@ void hnbgw_vty_init(void)
 	install_element(PFCP_NODE, &cfg_pfcp_local_addr_cmd);
 	install_element(PFCP_NODE, &cfg_pfcp_local_port_cmd);
 	install_element(PFCP_NODE, &cfg_pfcp_remote_addr_cmd);
+	install_element(PFCP_NODE, &cfg_pfcp_netinst_cmd);
 #endif
 
 	osmo_tdef_vty_groups_init(HNBGW_NODE, hnbgw_tdef_group);
